package xyz.zhouxy.plusone.oss;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;

import org.csource.common.MyException;
import org.csource.common.NameValuePair;
import org.csource.fastdfs.ClientGlobal;
import org.csource.fastdfs.FileInfo;
import org.csource.fastdfs.StorageClient;
import org.csource.fastdfs.StorageServer;
import org.csource.fastdfs.TrackerClient;
import org.csource.fastdfs.TrackerServer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.io.ClassPathResource;

import lombok.Getter;

public class FastDFSUtil {

    private static TrackerClient trackerClient;
    private static TrackerServer trackerServer;
    private static StorageServer storageServer;

    private static final Logger logger = LoggerFactory.getLogger(FastDFSUtil.class);

    private FastDFSUtil() {
        throw new IllegalStateException("Utility class");
    }

    static {
        try {
            String filePath = new ClassPathResource("fdfs_client.conf").getFile().getAbsolutePath();
            ClientGlobal.init(filePath);
            /*
             * TODO【重构】 将配置信息集成在 SpringBoot 配置文件中，使用加载 Properties 对象的方式进行配置，去除 fdfs_client.conf
             * Properties props = new Properties();
             * props.put(ClientGlobal.PROP_KEY_TRACKER_SERVERS, "10.0.11.101:22122,10.0.11.102:22122");
             * ClientGlobal.initByProperties(props);
             */
            trackerClient = new TrackerClient();
            trackerServer = trackerClient.getTrackerServer();
            storageServer = trackerClient.getStoreStorage(trackerServer);
        } catch (Exception e) {
            logger.error("FastDFS Client Init Fail!", e);
        }
    }

    /**
     * 上传文件到 Fast DFS，失败将抛异常
     *
     * @param file 文件信息
     * @return 2 elements string array if success:<br>
     *         <ul>
     *         <li>results[0]: the group name to store the file</li>
     *         </ul>
     *         <ul>
     *         <li>results[1]: the new created filename</li>
     *         </ul>
     *         return null if fail
     * @throws FastDFSException
     */
    public static String[] upload(FastDFSFile file) throws FastDFSException {
        logger.info("File Name: {}, File Length: {}", file.getName(), file.getContent().length);

        NameValuePair[] metaList = new NameValuePair[1];
        metaList[0] = new NameValuePair("author", file.getAuthor());

        long startTime = System.currentTimeMillis();
        StorageClient storageClient = null;
        String[] uploadResults = null;
        try {
            storageClient = new StorageClient(trackerServer, storageServer);
            uploadResults = storageClient.upload_file(file.getContent(), file.getExt(), metaList);

            if (uploadResults == null) {
                throw new FastDFSException("upload file fail, error code: " + storageClient.getErrorCode());
            }
            logger.info("Upload file successfully!!! group_name: {}, remoteFileName: {}, time used: {} ms",
                    uploadResults[0], uploadResults[1], System.currentTimeMillis() - startTime);

        } catch (IOException e) {
            throw new FastDFSException("IO Exception when uploadind the file:" + file.getName(), e);
        } catch (MyException e) {
            throw new FastDFSException(e);
        }

        return uploadResults;
    }

    public static FileInfo getFile(String groupName, String remoteFileName) throws FastDFSException {
        try {
            StorageClient storageClient = new StorageClient(trackerServer, storageServer);
            return storageClient.get_file_info(groupName, remoteFileName);
        } catch (IOException e) {
            throw new FastDFSException("IO Exception: Get File from Fast DFS failed", e);
        } catch (Exception e) {
            throw new FastDFSException("Non IO Exception: Get File from Fast DFS failed", e);
        }
    }

    public static InputStream downFile(String groupName, String remoteFileName) throws FastDFSException {
        try {
            StorageClient storageClient = new StorageClient(trackerServer, storageServer);
            byte[] fileByte = storageClient.download_file(groupName, remoteFileName);
            InputStream ins = new ByteArrayInputStream(fileByte);
            return ins;
        } catch (IOException e) {
            throw new FastDFSException("IO Exception: Get File from Fast DFS failed", e);
        } catch (Exception e) {
            throw new FastDFSException("Non IO Exception: Get File from Fast DFS failed", e);
        }
    }

    public static void deleteFile(String groupName, String remoteFileName) throws FastDFSException {
        StorageClient storageClient = new StorageClient(trackerServer, storageServer);
        try {
            int i = storageClient.delete_file(groupName, remoteFileName);
            if (i == 0) {
                logger.info("Delete file SUCCESSFULLY!!!");
            } else {
                throw new FastDFSException("Delete file failed, error code is: " + i);
            }
        } catch (IOException | MyException e) {
            throw new FastDFSException(e);
        }
    }

    @Getter
    public static final class FastDFSFile {
        private String name;
        private byte[] content;
        private String ext;
        private String md5;
        private String author;

        public FastDFSFile(String name, byte[] content, String ext) {
            this.name = name;
            this.content = content;
            this.ext = ext;
        }
    }
}
